/*
 * Copyright (c) 2009-2012 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/*
 * MEAS_KMA36_Main.c: Console Application for Testing the KMA36
 *
 * This application configures UART 16550 to baud rate 9600.
 * PS7 UART (Zynq) is not initialized by this application, since
 * bootrom/bsp configures it to baud rate 115200
 *
 * ------------------------------------------------
 * | UART TYPE   BAUD RATE                        |
 * ------------------------------------------------
 *   uartns550   9600
 *   uartlite    Configurable only in HW design
 *   ps7_uart    115200 (configured by bootrom/bsp)
 */

#include <stdio.h>
#include <unistd.h>
#include "platform.h"
#include "xparameters.h"
#include "kma36.h"

void kma36_main_menu(void);

int main()
{

    char key_input;
    u8 address_set_flag=0;
    kma36_status stat;
    float angle;
    u32 res=0;

    //Initialize the UART
    init_platform();

    // Set the AXI address of the IIC core and
    // initialize the i2c address to 0x77
    kma36_init(XPAR_AXI_IIC_JC_BASEADDR);

    // Display the main menu
    kma36_main_menu();

    // Infinite loop
    while(1){

        // Get keyboard input
        read(1, (char*)&key_input, 1);

        if(key_input == '1'){               //If the '1' key is pressed

            // Display address selection menu
            printf("\n");
            printf("Select an address:\n");
            printf("  (0)   - A0 is tied to GND     (Address=0x59)\n");
            printf("  (1)   - A0 is tied to DCOILP  (Address=0x5A)\n");
            printf("  (2)   - A0 is tied to DCOILN  (Address=0x5B)\n");
            printf("  (3)   - A0 is tied to DVCC_SE (Address=0x5C)\n");
            printf("  (4)   - A0 is tied to VCC     (Address=0x5D)\n");

            // Get keyboard input ignoring keypresses that are not '0' or '1' or '2' or '3' or '4'
            read(1, (char*)&key_input, 1);
            while(key_input!='0' && key_input!='1' && key_input!='2' && key_input!='3' && key_input!='4'){
                read(1, (char*)&key_input, 1);
            }

            if(key_input == '0'){           // If the '0' key is pressed
                // Set i2c address to 0x59
                kma36_set_i2c_address(kma36_i2c_address_GND);
                printf("Set KMA36 I2C Address to 0x59 (A0 tied to GND)\n");
            }else if(key_input == '1'){     // If the '1' key is pressed
                // Set i2c address to 0x5A
                kma36_set_i2c_address(kma36_i2c_address_DCOILP);
                printf("Set KMA36 I2C Address to 0x5A (A0 tied to DCOILP)\n");
            }else if(key_input == '2'){     // If the '2' key is pressed
                // Set i2c address to 0x5B
                kma36_set_i2c_address(kma36_i2c_address_DCOILN);
                printf("Set KMA36 I2C Address to 0x5B (A0 tied to DCOILN)\n");
            }else if(key_input == '3'){     // If the '3' key is pressed
                // Set i2c address to 0x5C
                kma36_set_i2c_address(kma36_i2c_address_DVCC_SE);
                printf("Set KMA36 I2C Address to 0x5C (A0 tied to DVCC_SE)\n");
            }else if(key_input == '4'){     // If the '4' key is pressed
                // Set i2c address to 0x5D
                kma36_set_i2c_address(kma36_i2c_address_VCC);
                printf("Set KMA36 I2C Address to 0x5D (A0 tied to VCC)\n");
            }

            address_set_flag = 1;
            printf("Reading initial register state...\n");
            stat = kma36_read_regs();
            if(stat==kma36_status_ok){
            	printf("Register read successful.\n");
            }else{
            	printf("Register read failed.\n");
            }
            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == '2'){         //If the '2' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				// Send the angle read command to the KMA36
				printf("\n");
				printf("Reading current angle from KMA36...\n");
				stat = kma36_read_angle(&angle);

				// Display the status returned from the angle read operation
				printf("KMA36 Angle Read Complete with status: ");
				if(stat==kma36_status_ok)
					printf("Ok.\n");
					printf("Angle: %4.1f%c\n",angle,248);
				if(stat==kma36_status_i2c_transfer_error)
					printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
			printf("\nPress any key to continue...\n");
			read(1, (char*)&key_input, 1);
			kma36_main_menu();


        }else if(key_input == '3'){         // If the '3' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				// Send request to KMA36 to enter sleep mode
				printf("\n");
				printf("KMA36 Entering Sleep Mode...\n");
				stat = kma36_sleep_enter();

				// Display status returned from enter sleep mode operation
				printf("Enter Sleep Mode Complete with status: ");
				if(stat==kma36_status_ok)
					printf("Ok.\n");
				if(stat==kma36_status_i2c_transfer_error)
					printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == '4'){         // If the '4' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				// Send request to KMA36 to exit sleep mode
				printf("\n");
				printf("KMA36 Exiting Sleep Mode...\n");
				stat = kma36_sleep_exit();

				// Display status returned from exit sleep mode operation
				printf("Exit Sleep Mode Complete with status: ");
				if(stat==kma36_status_ok)
					printf("Ok.\n");
				if(stat==kma36_status_i2c_transfer_error)
					printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == '5'){         // If the '5' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				// Send request to KMA36 to enable low power mode
				printf("\n");
				printf("KMA36 Enabling Low Power Mode...\n");
				stat = kma36_enable_low_power_mode();

				// Display status returned from enable low power operation
				printf("Enable Low Power Mode Complete with status: ");
				if(stat==kma36_status_ok)
					printf("Ok.\n");
				if(stat==kma36_status_i2c_transfer_error)
					printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == '6'){         // If the '6' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				// Send request to KMA36 to disable low power mode
				printf("\n");
				printf("KMA36 Disabling Low Power Mode...\n");
				stat = kma36_disable_low_power_mode();

				// Display status returned from disable low power operation
				printf("Disable Low Power Mode Complete with status: ");
				if(stat==kma36_status_ok)
					printf("Ok.\n");
				if(stat==kma36_status_i2c_transfer_error)
					printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == '7'){         // If the '7' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				// Send request to KMA36 to enable counter
				printf("\n");
				printf("KMA36 Enabling Counter...\n");
				stat = kma36_enable_counter();

				// Display status returned from enable counter operation
				printf("Enable Counter Complete with status: ");
				if(stat==kma36_status_ok)
					printf("Ok.\n");
				if(stat==kma36_status_i2c_transfer_error)
					printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == '8'){         // If the '8' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				// Send request to KMA36 to disable counter
				printf("\n");
				printf("KMA36 Disabling Counter...\n");
				stat = kma36_disable_counter();

				// Display status returned from disable counter operation
				printf("Disable Counter Complete with status: ");
				if(stat==kma36_status_ok)
					printf("Ok.\n");
				if(stat==kma36_status_i2c_transfer_error)
					printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == '9'){         // If the '9' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				// Send request to KMA36 to enable fast rate
				printf("\n");
				printf("KMA36 Enabling Fast Rate...\n");
				stat = kma36_enable_fast_rate();

				// Display status returned from enable fast rate operation
				printf("Enable Fast Rate Complete with status: ");
				if(stat==kma36_status_ok)
					printf("Ok.\n");
				if(stat==kma36_status_i2c_transfer_error)
					printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == '0'){         // If the '0' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				// Send request to KMA36 to disable fast rate
				printf("\n");
				printf("KMA36 Disabling Fast Rate...\n");
				stat = kma36_disable_fast_rate();

				// Display status returned from disable fast rate operation
				printf("Disable Fast Rate Complete with status: ");
				if(stat==kma36_status_ok)
					printf("Ok.\n");
				if(stat==kma36_status_i2c_transfer_error)
					printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == 'a' || key_input == 'A'){               //If the 'a' or 'A' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				// Display oversampling selection menu
				printf("\n");
				printf("Select an oversampling rate:\n");
				printf("  (0)   - Oversampling Rate 2\n");
				printf("  (1)   - Oversampling Rate 4\n");
				printf("  (2)   - Oversampling Rate 8\n");
				printf("  (3)   - Oversampling Rate 32\n");

				// Get keyboard input ignoring keypresses that are not '0' or '1' or '2' or '3'
				read(1, (char*)&key_input, 1);
				while(key_input!='0' && key_input!='1' && key_input!='2' && key_input!='3'){
					read(1, (char*)&key_input, 1);
				}

				if(key_input == '0'){           // If the '0' key is pressed
					// Set oversampling to 2
					kma36_set_accuracy(kma36_oversampling_2);
					printf("Set KMA36 Oversampling Rate to 2\n");
				}else if(key_input == '1'){     // If the '1' key is pressed
					// Set oversampling to 4
					kma36_set_accuracy(kma36_oversampling_4);
					printf("Set KMA36 Oversampling Rate to 4\n");
				}else if(key_input == '2'){     // If the '2' key is pressed
					// Set oversampling to 8
					kma36_set_accuracy(kma36_oversampling_8);
					printf("Set KMA36 Oversampling Rate to 8\n");
				}else if(key_input == '3'){     // If the '3' key is pressed
					// Set oversampling to 32
					kma36_set_accuracy(kma36_oversampling_32);
					printf("Set KMA36 Oversampling Rate to 32\n");
				}
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == 'b' || key_input == 'B'){         // If the 'b' or 'B' key is pressed

            if(address_set_flag==0){          // Address was not set yet--cannot perform this operation
                printf("KMA36 I2C Address has not yet been set.  Cannot complete this operation.\n");
            }else{
				res = 0;
				// If resolution is out of bounds, get a new number
				while(res<1 || res>32767){
					res = 0;
					// Display oversampling selection menu
					printf("\nSpecify a resolution between 1 and 32767:\n  ");
					// Get keyboard input ignoring keypresses that are not numbers or the enter key
					read(1, (char*)&key_input, 1);
					if(key_input=='0' || key_input=='1' || key_input=='2' || key_input=='3' || key_input=='4' || key_input=='5' || key_input=='6' || key_input=='7' || key_input=='8' || key_input=='9'){
						res *= 10;
						res += (key_input-0x30);
						printf("%c",key_input);
						fflush(stdout);
					}
					while(key_input!=(0x0D)){
						read(1, (char*)&key_input, 1);
						if(key_input=='0' || key_input=='1' || key_input=='2' || key_input=='3' || key_input=='4' || key_input=='5' || key_input=='6' || key_input=='7' || key_input=='8' || key_input=='9'){
							res *= 10;
							res += (key_input-0x30);
							printf("%c",key_input);
							fflush(stdout);
						}
					}
					if(res<1 || res>32767){
						printf("\n\nInvalid Resolution Value \"%u\".  Press any key to continue...\n",(unsigned int)res);
						read(1, (char*)&key_input, 1);
						kma36_main_menu();
					}else{
						kma36_set_resolution((u16)res);
						printf("\n\nSet Resolution to %u\n",(unsigned int)res);
					}
				}
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            kma36_main_menu();

        }else if(key_input == 27){    // If the 'ESC' key is pressed

            // Print done and exit.
            printf("Done.\n");
            break;

        }else{                      // If some other key is pressed

            // Redisplay the main menu
        	kma36_main_menu();

        }

    }

    return 0;

}

void kma36_main_menu(void){

    //Clear the screen
    printf("\033[2J");

    //Display the main menu
    printf("*******************************************\n");
    printf("****      Measurement Specialties      ****\n");
    printf("*******************************************\n");

    printf("\n");
    printf("    KMA36 - Universal Magnetic Encoder     \n");
    printf("-------------------------------------------\n");

    printf("\n");
    printf("Select a task:\n");
    printf("  (1)   - Set KMA36 I2C Address\n");
    printf("  (2)   - Read KMA36 Angle\n");
    printf("  (3)   - Enter Sleep Mode\n");
    printf("  (4)   - Exit Sleep Mode\n");
    printf("  (5)   - Enable Low Power Mode\n");
    printf("  (6)   - Disable Low Power Mode\n");
    printf("  (7)   - Enable Counter\n");
    printf("  (8)   - Disable Counter\n");
    printf("  (9)   - Enable Fast Rate\n");
    printf("  (0)   - Disable Fast Rate\n");
    printf("  (A)   - Set Accuracy\n");
    printf("  (B)   - Set Resolution\n");
    printf(" (ESC)  - Quit\n");
    printf("\n");

    return;
}
