/*
 * kma36.h
 *
 *  Created on: Jul 30, 2014
 *      Author: JMBrinkhus
 */

#include <stdio.h>
#include "xiic_l.h"


#ifndef KMA36_H_
#define KMA36_H_

#define CRC_POLY					(0x131)	// CRC Polynomial: X^8 + X^5 + X^4 + 1

// KMA36 BIT MASKS
#define KMA36_SLP				(0x80)
#define KMA36_LIN				(0x20)
#define KMA36_CNT				(0x10)
#define KMA36_PWR				(0x08)
#define KMA36_SPD				(0x04)
#define KMA36_OVSC				(0x03)
#define KMA36_OS2				(0x00)
#define KMA36_OS4				(0x01)
#define KMA36_OS8				(0x02)
#define KMA36_OS32				(0x03)

// KMA36 Enumerations
typedef enum kma36_address{
	kma36_i2c_address_GND = 	0x59,			// 1110_111x
	kma36_i2c_address_DCOILP = 	0x5A,			//
	kma36_i2c_address_DCOILN = 	0x5B,			//
	kma36_i2c_address_DVCC_SE = 0x5C,			//
	kma36_i2c_address_VCC = 	0x5D			//
}kma36_address;
typedef enum kma36_status{
	kma36_status_ok = 0,
	kma36_status_i2c_transfer_error = 1,
	kma36_status_crc_error = 2
}kma36_status;
typedef enum kma36_oversampling{
	kma36_oversampling_2 = 0,
	kma36_oversampling_4 = 1,
	kma36_oversampling_8 = 2,
	kma36_oversampling_32 = 3
}kma36_oversampling;

#endif /* KMA36_H_ */

// External variables
extern 	u32 			kma36_axi_address;
extern 	kma36_address 	kma36_i2c_address;
extern	u16				kma36_ma;
extern	u32				kma36_ilc;
extern	u8				kma36_kconf;
extern	u8				kma36_kresh;
extern	u8				kma36_kresl;

// Function Declarations
void kma36_init(u32 axi_address);
void kma36_set_i2c_address(kma36_address addr);
kma36_status kma36_read_angle(float* angle);
kma36_status kma36_sleep_enter(void);
kma36_status kma36_sleep_exit(void);
kma36_status kma36_enable_low_power_mode(void);
kma36_status kma36_disable_low_power_mode(void);
kma36_status kma36_enable_counter(void);
kma36_status kma36_disable_counter(void);
kma36_status kma36_enable_fast_rate(void);
kma36_status kma36_disable_fast_rate(void);
kma36_status kma36_set_accuracy(kma36_oversampling os);
kma36_status kma36_set_resolution(u16 res);
kma36_status kma36_read_regs(void);
u8 kma36_compute_crc(u8* write_regs);
kma36_status kma36_write_regs(void);
